#!/usr/bin/python3.6
"""Various tools to emerge and to show status for conky."""
from logging import debug, basicConfig, DEBUG, info
from time import strftime
from typing import List, Tuple

from pyerge import utils, tmplogfile, tmerge_logfile, dev_null

basicConfig(format='%(asctime)s | %(levelname)-6s | %(message)s', level=DEBUG)


def emerge(arguments: List[str], verbose: bool, build=True) -> bytes:
    """
    Run emerge command.

    :param arguments:
    :param verbose:
    :param build:
    :return:
    """
    if verbose:
        info(f"running emerge with: {' '.join(arguments)}")
    cmd = f"sudo /usr/bin/emerge --nospinner {' '.join(arguments)}"
    if build:
        return_code, _ = utils.run_cmd(cmd, use_system=True)
        return return_code
    output, _ = utils.run_cmd(cmd)
    return output


# <=><=><=><=><=><=><=><=><=><=><=><=> chk_upd <=><=><=><=><=><=><=><=><=><=><=><=>
def check_upd(local_chk: bool, verbose: bool) -> None:
    """
    Check system updates.

    :param local_chk:
    :param verbose:
    """
    utils.delete_content(tmplogfile)
    utils.delete_content(tmerge_logfile)
    tmp = open(tmplogfile, 'w')
    log = open(tmerge_logfile, 'w')
    tmp.write(strftime('%a %b %d %H:%M:%S %Z %Y') + '\n')
    if not local_chk:
        if verbose:
            # info('Start syncing overlays...')
            # utils.run_cmd(f'sudo layman -SN >> {tmplogfile} > {dev_null}', use_subproc=False)
            info('Start syncing portage...')
        if verbose > 1:
            debug(f'sudo eix-sync >> {tmplogfile} > {dev_null}')
        utils.run_cmd(f'sudo eix-sync >> {tmplogfile} > {dev_null}', use_system=True)
    if verbose:
        info('Checking updates...')
    output = emerge('-pvNDu --color n @world'.split(), verbose, build=False)
    log.write(output.decode(encoding='utf-8'))
    tmp.close()
    log.close()

    if verbose:
        info('Creating log file...')
    if verbose > 1:
        debug(f'cat {tmerge_logfile} >> {tmplogfile}')
    utils.run_cmd(f'cat {tmerge_logfile} >> {tmplogfile}', use_system=True)
    if verbose > 1:
        debug(f'cat {tmerge_logfile} | genlop -pn >> {tmplogfile}')
    utils.run_cmd(f'cat {tmerge_logfile} | genlop -pn >> {tmplogfile}', use_system=True)


# <=><=><=><=><=><=><=><=><=><=><=><=> tmerge <=><=><=><=><=><=><=><=><=><=><=><=>
def post_emerge(args: List[str], verbose: bool, return_code: bytes) -> None:
    """
    Run actions after emerge.

    :param args:
    :param verbose:
    :param return_code:
    """
    pretend, world = check_emerge_opts(args)
    if not int(return_code) and not pretend and world:
        if verbose:
            info('Clearing emerge log')
        tmp = open(tmplogfile, 'w')
        log = open(tmerge_logfile, 'w')
        log.write('Total: 0 packages, Size of downloads: 0 KiB')
        tmp.close()
        log.close()


def deep_clean(args: List[str], verbose: bool, return_code: bytes) -> None:
    """
    Run deep clean after emerge.

    :param args:
    :param verbose:
    :param return_code:
    """
    pretend, world = check_emerge_opts(args)
    if not int(return_code) and not pretend and world:
        out = emerge(['-pc'], verbose, build=False)
        if verbose:
            info('Deep clean')
        if verbose > 1:
            debug(f'Details:{out.decode(encoding="utf-8")}')


def check_emerge_opts(args: List[str]) -> Tuple[bool, bool]:
    """
    Check options in emerge command.

    :param args:
    :return:
    """
    pretend = True
    world = False
    if 'p' not in args[0] or 'f' in args[0]:
        pretend = False
    if 'world' in ' '.join(args):
        world = True
    return pretend, world


def is_portage_running() -> bool:
    """
    Check if potrage command in currently running.

    :return: True if is running, False otherwise
    """
    out, _ = utils.run_cmd('pgrep -f /usr/bin/emerge')
    return bool(out)



#!/usr/bin/python3.6
"""Various tools to emerge and to show status for conky."""
from re import search

from pyerge import utils

__version__ = '0.3'


if __name__ == '__main__':
    output, _ = utils.run_cmd('genlop -cn')
    if search(r'Error.*no working merge found', output.decode()):
        print('Unknown')
        exit()

    match = search(r'ETA:\s(.*)\.', output.decode())
    if match:
        print(match.group(1))



from unittest import mock

from pytest import mark

from pyerge import tmerge


def test_is_portage_running():
    with mock.patch('pyerge.tmerge.utils') as utils_mock:
        utils_mock.run_cmd.return_value = (b'3456\n', b'')
        assert tmerge.is_portage_running() is True
        utils_mock.run_cmd.assert_called_once_with('pgrep -f /usr/bin/emerge')


def test_is_portage_not_running():
    with mock.patch('pyerge.tmerge.utils') as utils_mock:
        utils_mock.run_cmd.return_value = (b'', b'')
        assert tmerge.is_portage_running() is False
        utils_mock.run_cmd.assert_called_once_with('pgrep -f /usr/bin/emerge')


@mark.parametrize('list_str, result', [(['-pvNDu', '@world'], (True, True)),
                                       (['-pv', 'conky'], (True, False)),
                                       (['-f', 'conky'], (False, False)),
                                       (['', '@world'], (False, True)),
                                       (['', 'conky'], (False, False))])
def test_check_emerge_opts(list_str, result):
    assert tmerge.check_emerge_opts(list_str) == result



#!/usr/bin/python3.6
"""Various tools to emerge and to show status for conky."""
from datetime import datetime
from re import search

from pyerge import emerge_logfile, tmerge_logfile


def e_sync() -> str:
    """
    Fetch date of last sync form logs.

    :return: date as string
    """
    with open(emerge_logfile) as log_file:
        for line in reversed(list(log_file)):
            match = search(r'(\d+)(:\s===\sSync completed)', line)
            if match is not None:
                sync_time = match.group(1)
                break
        else:
            return 'Unknown'
    return datetime.fromtimestamp(int(sync_time)).strftime('%A %H:%M')


def e_dl() -> str:
    """
    Fetch size of archives to be download for next system update.

    :return: date as string
    """
    size = None

    with open(tmerge_logfile, 'r') as log:
        match = search(r'(Size of downloads:.)([0-9,]*\s[KMG]iB)', str(log.readlines()))
        if match is not None:
            size = match.group(2)

    if size == '0 KiB':
        size = 'None'
    elif not size:
        size = 'Calculating...'

    return size



from sys import version_info
from unittest import mock
from unittest.mock import mock_open, Mock

from pytest import mark

from pyerge.file_utils import e_sync

str_no_sync = """1548194358:  === sync
1548194358: >>> Syncing repository 'gentoo' into '/usr/portage'...
1548194361: >>> Starting rsync with rsync://[2a01:90:200:10::1a]/gentoo-portage"""

str_sync = """1548194361: >>> Starting rsync with rsync://[2a01:90:200:10::1a]/gentoo-portage
1548194466: === Sync completed for gentoo
1548194466:  *** terminating."""


def test_synced():
    with mock.patch('pyerge.file_utils.open') as open_mock:
        open_mock.return_value.__enter__ = open_mock
        open_mock.return_value.__iter__ = Mock(return_value=iter(list(str_sync.split('\n'))))
        assert e_sync() == 'Tuesday 22:01' or 'Tuesday 23:01'


def test_no_synced():
    with mock.patch('pyerge.file_utils.open') as open_mock:
        open_mock.return_value.__enter__ = open_mock
        open_mock.return_value.__iter__ = Mock(return_value=iter(list(str_no_sync.split('\n'))))
        assert e_sync() == 'Unknown'


@mark.skipif(condition=version_info < (3, 7), reason='Run only on Python 3.7+')
def test_synced_ver2():
    with mock.patch('pyerge.file_utils.open', mock_open(read_data=str_sync)) as m:
        result = e_sync()

    m.assert_called_once_with('/var/log/emerge.log')
    assert result == 'Tuesday 22:01' or 'Tuesday 23:01'



#!/usr/bin/python3.6
"""Various tools to emerge and to show status for conky."""
from enum import Enum
from logging import debug, basicConfig, DEBUG, info
from os import environ, system
from time import strftime
from typing import List, Tuple

from pyerge import utils, portage_tmpdir, tmplogfile, tmerge_logfile, dev_null

basicConfig(format='%(asctime)s | %(levelname)-6s | %(message)s', level=DEBUG)


class World(Enum):
    PRETEND = ['-pvNDu', '@world']
    WORLD = ['-NDu', '@world']


class Tmerge:
    def __init__(self, world: World, verbose=False, deep_clean=False) -> None:
        self.world = world
        self.verbose = verbose
        self.deep_clean = deep_clean

    def emerge(self, arguments: List[str], verbose: bool, build=True) -> bytes:
        """
        Run emerge command.

        :param arguments:
        :param verbose:
        :param build:
        :return:
        """
        if verbose:
            info(f"running emerge with: {' '.join(arguments)}")
        if build:
            return_code = system(f"sudo /usr/bin/emerge --nospinner {' '.join(arguments)}")
            return bytes(return_code)
        else:
            output, _ = utils.run_cmd(f"sudo /usr/bin/emerge --nospinner {' '.join(arguments)}")
            return output

    def check(self, local_chk: bool, verbose: bool) -> None:
        """
        Check system updates.

        :param local_chk:
        :param verbose:
        """
        utils.delete_content(tmplogfile)
        utils.delete_content(tmerge_logfile)
        tmp = open(tmplogfile, 'w')
        log = open(tmerge_logfile, 'w')
        tmp.write(strftime('%a %b %d %H:%M:%S %Z %Y') + '\n')
        if not local_chk:
            if verbose:
                # info('Start syncing overlays...')
                # system(f'sudo layman -SN >> {tmplogfile} > {DEVNULL}')
                info('Start syncing portage...')
            if verbose > 1:
                debug(f'sudo eix-sync >> {tmplogfile} > {dev_null}')
            system(f'sudo eix-sync >> {tmplogfile} > {dev_null}')
            if verbose:
                info('Checking updates...')
        output = self.emerge('-pvNDu --color n @world'.split(), verbose, build=False)
        if verbose:
            info('Updates checked')
        log.write(output.decode(encoding='utf-8'))
        tmp.close()
        log.close()

        # system('sudo /usr/local/sbin/tmerge.py 1G -pvNDu --with-bdeps=y --color n @world >> %s' % logfile)
        if verbose:
            info('Creating log file...')
        if verbose > 1:
            debug(f'cat {tmerge_logfile} >> {tmplogfile}')
        system(f'cat {tmerge_logfile} >> {tmplogfile}')
        if verbose > 1:
            debug(f'cat {tmerge_logfile} | genlop -pn >> {tmplogfile}')
        system(f'cat {tmerge_logfile} | genlop -pn >> {tmplogfile}')
        if verbose:
            info('Wrote to logs file')

    def is_running(self) -> bool:
        """
        Check if potrage command in currently running.

        :return: True if is running, False otherwise
        """
        if utils.run_cmd('pgrep -f /usr/bin/emerge'):
            return True
        return False

    def post_emerge(self, args: List[str], verbose: bool, return_code: bytes) -> None:
        """
        Run actions after emerge.

        :param args:
        :param verbose:
        :param return_code:
        """
        pretend, world = self.check_emerge_opts(args)
        if not int(return_code) and not pretend and world:
            if verbose:
                info('Clearing emerge log')
            tmp = open(tmplogfile, 'w')
            log = open(tmerge_logfile, 'w')
            log.write('Total: 0 packages, Size of downloads: 0 KiB')
            tmp.close()
            log.close()

    def deep_clean(self, args: List[str], verbose: bool, return_code: bytes) -> None:
        """
        Run deep clean after emerge.

        :param args:
        :param verbose:
        :param return_code:
        """
        pretend, world = self.check_emerge_opts(args)
        if not int(return_code) and not pretend and world:
            out = self.emerge(['-pc'], verbose, build=False)
            if verbose:
                info('Deep clean')
            if verbose > 1:
                debug(f'Details:{out.decode(encoding="utf-8")}')

    def check_emerge_opts(self, args: List[str]) -> Tuple[bool, bool]:
        """
        Check options in emerge command.

        :param args:
        :return:
        """
        pretend = True
        world = False
        if 'p' not in args[0] or 'f' in args[0]:
            pretend = False
        if 'world' in ' '.join(args):
            world = True
        return pretend, world

    @staticmethod
    def set_portage_tmpdir() -> None:
        """Set system variable."""
        if environ.get('PORTAGE_TMPDIR') is None:
            environ['PORTAGE_TMPDIR'] = portage_tmpdir



"""Various tools to emerge and to show status for conky."""
from logging import warning, info, debug
from os import system, environ
from re import search
from shlex import split
from subprocess import Popen, PIPE  # nosec
from typing import Union, Tuple

from pyerge import server, portage_tmpdir


def run_cmd(cmd: str, use_system=False) -> Tuple[bytes, bytes]:
    """
    Run any system command.

    If use_system is set cmd is run via os.system and function
    return RC from comand as bytes and b''.
    If use_system is not set (default) cmd is run via subprocess.Popen and
    function return cmd stdout and stderr as bytes.

    :param cmd: command string
    :param use_system: os.system use insted of subprocess
    :return: tuple of bytes with output and error
    """
    if use_system:
        ret_code = system(cmd)  # nosec
        out, err = str(ret_code).encode(), b''
    else:
        out, err = Popen(split(cmd), stdout=PIPE, stderr=PIPE).communicate()  # nosec
    return out, err


def mounttmpfs(size: str, verbose: bool) -> None:
    """
    Mount directory with size as tmp file system in RAM.

    :param size: with unit K, M, G
    :param verbose: be verbose
    """
    if verbose:
        info(f'Mounting {size} of memory to {portage_tmpdir}')
    if verbose > 1:
        debug(f'sudo mount -t tmpfs -o size={size},nr_inodes=1M tmpfs {portage_tmpdir}')
    run_cmd(f'sudo mount -t tmpfs -o size={size},nr_inodes=1M tmpfs {portage_tmpdir}')


def unmounttmpfs(size: str, verbose: bool) -> None:
    """
    Unmount directory from RAM.

    :param size: with unit K, M, G
    :param verbose: be verbose
    """
    if verbose:
        info(f'Unmounting {size} of memory from {portage_tmpdir}')
    if verbose > 1:
        debug(f'sudo umount -f {portage_tmpdir}')
    run_cmd(f'sudo umount -f {portage_tmpdir}')


def remounttmpfs(size: str, verbose: bool) -> None:
    """
    Re-mount directory with size as tmp file system in RAM.

    :param size: with unit K, M, G
    :param verbose: be verbose
    """
    if verbose:
        info(f'Remounting {size} of memory to {portage_tmpdir}')
    if verbose > 1:
        debug(f'sudo umount -f {portage_tmpdir}')
    run_cmd(f'sudo umount -f {portage_tmpdir}')
    if verbose > 1:
        debug(f'sudo mount -t tmpfs -o size={size},nr_inodes=1M tmpfs {portage_tmpdir}')
    run_cmd(f'sudo mount -t tmpfs -o size={size},nr_inodes=1M tmpfs {portage_tmpdir}')


def is_internet_connected(verbose: bool) -> bool:
    """
    Check if there is connection to internet.

    :param verbose: be verbose
    :return: True is connected, False otherwise
    """
    cmd, _ = run_cmd(f'ping -W1 -c1 {server}')
    match = search(b'[1].*, [1].*, [0]%.*,', cmd)
    if match is not None:
        if verbose:
            info('There is internet connecton or not needed')
        return True
    warning('No internet connection!')
    return False


def size_of_mounted_tmpfs() -> int:
    """
    Return size of mounted directory.

    :return: size in bytes as intiger
    """
    df_cmd, _ = run_cmd('df')
    match = search(r'(tmpfs\s*)(\d+)(\s*.*%s)' % portage_tmpdir, df_cmd.decode())
    if match is not None:
        return int(match.group(2))
    return 0


def is_tmpfs_mounted() -> bool:
    """
    Check if portage temp dir is mounted.

    :return: True is mounted, False otherwise
    """
    mount_cmd, _ = run_cmd('mount')
    match = search(r'(tmpfs on\s+)(%s)(\s+type tmpfs)' % portage_tmpdir, mount_cmd.decode())
    return bool(match is not None and match.group(2) == portage_tmpdir)


def convert2blocks(size: str) -> int:
    """
    Convert size with unit into system blocks (used in i.e. df/mounts commands).

    :param size: with units K, M, G
    :return: size in kB
    """
    try:
        return int(float(size))
    except ValueError:
        pass
    match = search(r'(?i)(\d+)([KMG])', size)
    if match.group(2).upper() == 'K':
        # todo: add handling of floting point
        return int(match.group(1))
    if match.group(2).upper() == 'M':
        return int(match.group(1)) * 1024
    if match.group(2).upper() == 'G':
        return int(match.group(1)) * 1024 * 1024


def delete_content(fname: Union[str, bytes, int]) -> None:
    """
    Clean-up file content.

    :param fname: path to file as string
    """
    with open(fname, 'w'):
        pass


def set_portage_tmpdir() -> str:
    """Set system variable."""
    if not environ.get('PORTAGE_TMPDIR', ''):
        environ['PORTAGE_TMPDIR'] = portage_tmpdir
    return environ['PORTAGE_TMPDIR']



#!/usr/bin/python3.6
"""Various tools to emerge and to show status for conky."""
from argparse import ArgumentParser, Namespace
from logging import basicConfig, DEBUG, info, error
from typing import List

from pyerge import tmerge, utils, glsa, __version__

basicConfig(format='%(asctime)s | %(levelname)-6s | %(message)s', level=DEBUG)


def run_parser():
    """
    Function to collect command line arguments.

    Construct main object with correct set of parameters.
    """
    parser = ArgumentParser(description='Emerge in temporary RAM disk')
    parser.add_argument('-s', '--size', action='store', dest='size',
                        default='4G', help='Size or RAM disk, default 4G')
    parser.add_argument('-l', '--check_local', action='store_true', dest='local',
                        default=False, help='check locally')
    parser.add_argument('-d', '--deep_clean', action='store_true', dest='deep',
                        default=False, help='run deep clean after emerge')
    parser.add_argument('-w', '--world', action='store_true', dest='world',
                        default=False, help='run emerge -NDu @world')
    parser.add_argument('-r', '--pretend_world', action='store_true', dest='pretend_world',
                        default=False, help='run emerge -pvNDu @world')
    parser.add_argument('-v', '--verbose', action='count', dest='verbose',
                        help='Increase output verbosity')
    parser.add_argument('-e', '--elements', action='store', dest='elements', type=int,
                        default='5', help='number of elements')
    parser.add_argument('-V', '--version', action='version', version='%(prog)s ' + __version__)
    parser.add_argument('action', help='check, emerge, glsa_list or glsa_test')
    opts, emerge_opts = parser.parse_known_args()
    if opts.action not in ['check', 'emerge', 'glsa_list', 'glsa_test']:
        error(f'Wrong options: {opts} {emerge_opts}')
        exit()

    if opts.world:
        emerge_opts = ['-NDu', '@world']
    if opts.pretend_world:
        emerge_opts = ['-pvNDu', '@world']

    info(f'Pyerge version: {__version__}')
    opts.online = utils.is_internet_connected(opts.verbose)
    if opts.action in ('glsa_list', 'glsa_test'):
        run_glsa(opts)
        exit()

    if not tmerge.is_portage_running():
        utils.set_portage_tmpdir()
        handling_mounting(opts)
        run_emerge(emerge_opts, opts)
        run_check(opts)
        utils.unmounttmpfs(opts.size, opts.verbose)
    else:
        if opts.verbose:
            info('emerge already running!')


def handling_mounting(opts: Namespace) -> None:
    """
    Handling mounting temporary file fistem with requestes size.

    :param opts: cli arguments
    """
    if not opts.local:
        if not utils.is_tmpfs_mounted():
            utils.mounttmpfs(opts.size, opts.verbose)
        elif utils.size_of_mounted_tmpfs() != utils.convert2blocks(opts.size):
            utils.remounttmpfs(opts.size, opts.verbose)
        else:
            if opts.verbose:
                info('tmpfs is already mounted with requested size!')


def run_emerge(emerge_opts: List[str], opts: Namespace) -> None:
    """
    Run update of system.

    :param emerge_opts: list of arguments for emege
    :param opts: cli arguments
    """
    if opts.action == 'emerge' and opts.online:
        ret_code = tmerge.emerge(emerge_opts, opts.verbose, build=True)
        tmerge.post_emerge(emerge_opts, opts.verbose, ret_code)
        if opts.deep:
            tmerge.deep_clean(emerge_opts, opts.verbose, ret_code)


def run_check(opts: Namespace) -> None:
    """
    Run checking system updates.

    :param opts: cli arguments
    """
    if opts.action == 'check' and (opts.online or opts.local):
        tmerge.check_upd(opts.local, opts.verbose)


def run_glsa(opts: Namespace) -> None:
    """
    Run gGLSA module to test or to list.

    :param opts: cli arguments
    """
    if opts.online:
        try:
            attr = getattr(glsa, opts.action)
        except AttributeError:
            pass
        else:
            print(attr(opts))



"""Various tools to emerge and to show status for conky."""
import io
from os.path import abspath, dirname, join

from setuptools import setup, find_packages

from pyerge import __version__

here = abspath(dirname(__file__))

with io.open(join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

with io.open(join(here, 'requirements.txt'), encoding='utf-8') as f:
    requires = f.read().splitlines()

setup(
    name='pyerge',  # Required
    version=__version__,  # Required
    description='Various tools to emerge and to show status for conky',  # Required
    long_description=long_description,  # Optional
    long_description_content_type='text/markdown',  # Optional (see note above)
    url='https://github.com/emcek/pyerge',  # Optional
    author='Michal Plichta',  # Optional
    license='GPLv2',
    scripts=['script/pye'],
    # entry_points={'console_scripts': ['exec = pyerge.cli:run_parser']},
    classifiers=[  # Optional
        'Development Status :: 4 - Beta',

        'Environment :: Console',
        'Environment :: X11 Applications',

        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',

        'Operating System :: POSIX',
        'Operating System :: Unix',

        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',

        'Topic :: Desktop Environment',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: System :: Monitoring',
        'Topic :: Utilities'
    ],

    keywords='gentoo portage emerge conky',  # Optional

    # You can just specify package directories manually here if your project is
    # simple. Or you can use find_packages().
    #
    # Alternatively, if you just want to distribute a single Python file, use
    # the `py_modules` argument instead as follows, which will expect a file
    # called `my_module.py` to exist:
    #
    #   py_modules=["my_module"],
    #
    packages=find_packages(exclude=['tests']),  # Required

    # This field lists other packages that your project depends on to run.
    # Any package you put here will be installed by pip when your project is
    # installed, so they must be valid existing projects.
    #
    # For an analysis of "install_requires" vs pip's requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=requires,  # Optional

    extras_require={
        'testing': ['pytest']
    },

    project_urls={  # Optional
        'Bug Reports': 'https://github.com/emcek/pyerge/issues',
        'Source': 'https://github.com/emcek/pyerge',
    },
)



"""Various tools to emerge and to show status for conky."""
__version__ = '0.3.5'

portage_tmpdir = '/var/tmp/portage'  # nosec
tmplogfile = '/var/log/portage/tmerge/tmp.emerge.log'
tmerge_logfile = '/var/log/portage/tmerge/emerge.log'
emerge_logfile = '/var/log/emerge.log'
server = '89.16.167.134'
dev_null = '/dev/null 2>&1'
glsa_webpage = 'https://security.gentoo.org/glsa/feed.rss1'



#!/usr/bin/python3.6
"""Various tools to emerge and to show status for conky."""
from argparse import Namespace
from re import match
from typing import List
from urllib.request import urlopen

from bs4 import BeautifulSoup

from pyerge import utils, glsa_webpage


def glsa_list(opts: Namespace) -> str:
    """
    List GLSAs with number and name as string with new lines.

    :param opts: cli arguments
    :return: string with new lines
    """
    return '\n'.join(_rss(regex=r'GLSA\s(\d{6}-\d{2}:\s.*)', elements=opts.elements))


def glsa_test(opts: Namespace) -> str:
    """
    Test system against GLSAs.

    :param opts: cli arguments
    :return: string with result
    """
    glsalist = ' '.join(_rss(regex=r'GLSA\s(\d{6}-\d{2}):\s.*', elements=opts.elements))
    out, err = utils.run_cmd(f'glsa-check -t {glsalist}')
    if err == b'This system is not affected by any of the listed GLSAs\n':
        return 'System is not affected by any of listed GLSAs'
    return out.decode('UTF-8').strip().replace('\n', ',')


def _rss(regex: str, elements: int) -> List[str]:
    """
    Parse web page and find all matching tags.

    :param regex: regural expresion
    :param elements: number of elements to return
    :return: list of strings
    """
    with urlopen(glsa_webpage) as rss_page:  # nosec
        rss_html = rss_page.read().decode('UTF-8')
    all_versions = _collect_all_maching_entries(rss_html, regex)
    return all_versions[0:elements]


def _collect_all_maching_entries(html: str, regex: str) -> List[str]:
    """
    Parse web page and find all matching tags.

    :param html: web page
    :param regex: regural expresion
    :return: list of strings
    """
    tmp_list = []
    soup = BeautifulSoup(html, 'html.parser')
    all_a_tags = soup.find_all('title')
    for tag in all_a_tags:
        found = match(regex, tag.text)
        if found:
            tmp_list.append(found.group(1))
    return tmp_list



from unittest import mock

from pytest import mark

date1 = '201904-23'
date2 = '201904-22'
date3 = '201904-13'
date4 = '201904-14'
vulnerabilities = f'{date1}: GLib: Multiple vulnerabilities'
escalation = f'{date2}: OpenDKIM: Root privilege escalation'


def test_glsa_list():
    from pyerge.glsa import glsa_list
    from argparse import Namespace
    with mock.patch('pyerge.glsa._rss') as rss_mock:
        rss_mock.return_value = [vulnerabilities, escalation]
        assert glsa_list(Namespace(elements=2)) == f'{vulnerabilities}\n{escalation}'


def test_glsa_test_system_not_affected():
    from pyerge.glsa import glsa_test
    from argparse import Namespace
    with mock.patch('pyerge.glsa.utils') as utils_mock:
        with mock.patch('pyerge.glsa._rss') as rss_mock:
            rss_mock.return_value = [date1, date2]
            utils_mock.run_cmd.return_value = b'', b'This system is not affected by any of the listed GLSAs\n'
            assert glsa_test(Namespace(elements=2)) == 'System is not affected by any of listed GLSAs'


def test_glsa_test_system_affected():
    from pyerge.glsa import glsa_test
    from argparse import Namespace
    with mock.patch('pyerge.glsa.utils') as utils_mock:
        with mock.patch('pyerge.glsa._rss') as rss_mock:
            rss_mock.return_value = [date1, date2, date3]
            utils_mock.run_cmd.return_value = b'201904-13\n201904-14\n', b'This system is affected by the following GLSAs:\n'
            assert glsa_test(Namespace(elements=2)) == f'{date3},{date4}'


def test_rss():
    from pyerge.glsa import _rss
    html = b'<?xml version="1.0" encoding="UTF-8"?>\n<rdf:RDF xmlns="http://purl.org/rss/1.0/"\n  ' \
           b'xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"\n  ' \
           b'xmlns:content="http://purl.org/rss/1.0/modules/content/"\n  ' \
           b'xmlns:dc="http://purl.org/dc/elements/1.1/"\n  ' \
           b'xmlns:image="http://purl.org/rss/1.0/modules/image/"\n  ' \
           b'xmlns:slash="http://purl.org/rss/1.0/modules/slash/"\n  ' \
           b'xmlns:sy="http://purl.org/rss/1.0/modules/syndication/"\n  ' \
           b'xmlns:taxo="http://purl.org/rss/1.0/modules/taxonomy/"\n  ' \
           b'xmlns:trackback="http://madskills.com/public/xml/rss/module/trackback/">\n' \
           b'<channel rdf:about="https://security.gentoo.org/glsa">\n    ' \
           b'<title>Gentoo Linux security advisories</title>\n    ' \
           b'<link>https://security.gentoo.org/glsa</link>\n    ' \
           b'<description>This feed contains new Gentoo Linux security advisories. ' \
           b'Contact security@gentoo.org with questions.</description>\n' \
           b'<items>\n      <rdf:Seq>\n' \
           b'<rdf:li resource="https://security.gentoo.org/glsa/201904-25"/>\n' \
           b'<rdf:li resource="https://security.gentoo.org/glsa/201904-24"/>\n' \
           b'<rdf:li resource="https://security.gentoo.org/glsa/201904-23"/>\n' \
           b'</items>\n    <dc:date>2019-04-24T00:00:00+00:00</dc:date>\n  </channel>\n' \
           b'<item rdf:about="https://security.gentoo.org/glsa/201904-25">\n    ' \
           b'<title>GLSA 201904-25: QEMU: Multiple vulnerabilities</title>\n    ' \
           b'<link>https://security.gentoo.org/glsa/201904-25</link>\n    ' \
           b'<dc:date>2019-04-24T00:00:00+00:00</dc:date>\n  </item>\n' \
           b'<item rdf:about="https://security.gentoo.org/glsa/201904-24">\n    ' \
           b'<title>GLSA 201904-24: Ming: Multiple vulnerabilities</title>\n    ' \
           b'<link>https://security.gentoo.org/glsa/201904-24</link>\n    ' \
           b'<dc:date>2019-04-24T00:00:00+00:00</dc:date>\n  </item>\n' \
           b'<item rdf:about="https://security.gentoo.org/glsa/201904-23">\n    ' \
           b'<title>GLSA 201904-23: GLib: Multiple vulnerabilities</title>\n    ' \
           b'<link>https://security.gentoo.org/glsa/201904-23</link>\n    ' \
           b'<dc:date>2019-04-22T00:00:00+00:00</dc:date>\n  </item>\n' \
           b'</rdf:RDF>'

    with mock.patch('pyerge.glsa.urlopen') as urlopen_mock:
        with mock.patch('pyerge.glsa._collect_all_maching_entries') as collect_all_mock:
            urlopen_mock.return_value.read.return_value = html
            collect_all_mock.return_value = ['201904-25', '201904-24', '201904-23']
            assert _rss(r'GLSA\s(\d{6}-\d{2}):\s.*', 2) == ['201904-25', '201904-24']


@mark.parametrize('regex, result', [(r'GLSA\s(\d{6}-\d{2}):\s.*', [date1, date2]),
                                    (r'GLSA\s(\d{6}-\d{2}:\s.*)', [vulnerabilities, escalation])])
def test_collect_all_maching_entries(regex, result):
    from pyerge.glsa import _collect_all_maching_entries
    html = """
<?xml version="1.0" encoding="UTF-8"?>
<rdf:RDF xmlns="http://purl.org/rss/1.0/"
  xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
  xmlns:content="http://purl.org/rss/1.0/modules/content/"
  xmlns:dc="http://purl.org/dc/elements/1.1/"
  xmlns:image="http://purl.org/rss/1.0/modules/image/"
  xmlns:slash="http://purl.org/rss/1.0/modules/slash/"
  xmlns:sy="http://purl.org/rss/1.0/modules/syndication/"
  xmlns:taxo="http://purl.org/rss/1.0/modules/taxonomy/"
  xmlns:trackback="http://madskills.com/public/xml/rss/module/trackback/">
  <channel rdf:about="https://security.gentoo.org/glsa">
    <title>Gentoo Linux security advisories</title>
    <link>https://security.gentoo.org/glsa</link>
    <description>This feed contains new Gentoo Linux security advisories. Contact security@gentoo.org with questions.</description>
    <items>
      <rdf:Seq>
        <rdf:li resource="https://security.gentoo.org/glsa/201904-23"/>
        <rdf:li resource="https://security.gentoo.org/glsa/201904-22"/>
      </rdf:Seq>
    </items>
    <dc:date>2019-04-24T00:00:00+00:00</dc:date>
  </channel>
  <item rdf:about="https://security.gentoo.org/glsa/201904-23">
    <title>GLSA 201904-23: GLib: Multiple vulnerabilities</title>
    <link>https://security.gentoo.org/glsa/201904-23</link>
    <dc:date>2019-04-22T00:00:00+00:00</dc:date>
  </item>
  <item rdf:about="https://security.gentoo.org/glsa/201904-22">
    <title>GLSA 201904-22: OpenDKIM: Root privilege escalation</title>
    <link>https://security.gentoo.org/glsa/201904-22</link>
    <dc:date>2019-04-22T00:00:00+00:00</dc:date>
  </item>
</rdf:RDF>
"""
    assert _collect_all_maching_entries(html, regex) == result
